package com.elitesland.router.config;

import com.alibaba.nacos.api.config.ConfigService;
import com.alibaba.nacos.api.config.listener.Listener;
import com.elitesland.router.bean.GatewayFilterDefinition;
import com.elitesland.router.bean.GatewayPredicateDefinition;
import com.elitesland.router.bean.GatewayRouteDefinition;
import com.elitesland.router.service.DynamicRouteService;
import com.elitesland.router.util.GateWayNacosUtils;
import com.google.gson.Gson;
import com.google.gson.JsonArray;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.CommandLineRunner;
import org.springframework.cloud.gateway.filter.FilterDefinition;
import org.springframework.cloud.gateway.handler.predicate.PredicateDefinition;
import org.springframework.cloud.gateway.route.InMemoryRouteDefinitionRepository;
import org.springframework.cloud.gateway.route.RouteDefinition;
import org.springframework.stereotype.Component;
import org.springframework.web.util.UriComponentsBuilder;
import reactor.core.publisher.Flux;
import java.net.URI;
import java.util.ArrayList;
import java.util.concurrent.Executor;

/*
 *@Author Horbin.Zhang
 *@Date 2021-02-12 19:47
 *@Description 注册nacos监听，该类实现CommandLineRunner接口
 */

@Component
public class NacosConfigListener implements CommandLineRunner {
    @Autowired
    private GateWayNacosUtils gateWayNacosUtils;
    @Autowired
    private DynamicRouteService dynamicRouteService;
    @Autowired
    private InMemoryRouteDefinitionRepository inMemoryRouteDefinitionRepository;
    @Override
    public void run(String... args) {
        getConfig();
    }

    private void getConfig() {
        try {
            String routeDefinitions = gateWayNacosUtils.getRouteDefinitionsFromNacosServer();
            ConfigService configService = gateWayNacosUtils.getNacosConfigService();
            Gson gson = new Gson();
            //注册nacos配置监听，当nacos配置信息发生改变时刷新本地内存中Gateway Route规则信息
            refreshLocalRoute(routeDefinitions, gson, true);
            String group = gateWayNacosUtils.getNacosConfigGroup();
            String routeConfigDataId = gateWayNacosUtils.getNacosConfigDataIdForGateWayDefinition();
            configService.addListener(routeConfigDataId, group, new Listener() {
                @Override
                public void receiveConfigInfo(String configInfo) {
                    refreshLocalRoute(configInfo, gson, false);
                }
                @Override
                public Executor getExecutor() {
                    return null;
                }
            });
        } catch (Exception e) {
            getConfig();
            e.printStackTrace();
        }
    }

    /**
     * 根据nacos配置信息，刷新本地Gateway RouteDefinition
     */
    private void refreshLocalRoute(String initConfigInfo, Gson gson, boolean t) {
        //删除内存中已存在的动态路由配置信息
        Flux<RouteDefinition> routeDefinitions = inMemoryRouteDefinitionRepository.getRouteDefinitions();
        //由于不存在空指针情况，因此无需空指针判断
        routeDefinitions.subscribe(routeDefinition -> {
            dynamicRouteService.delete(routeDefinition.getId());
        });
        JsonArray jsonArray = gson.fromJson(initConfigInfo, JsonArray.class);
        jsonArray.forEach(routeDefinitionAsJsonObject -> {
            GatewayRouteDefinition gatewayRouteDefinition = gson.fromJson(routeDefinitionAsJsonObject, GatewayRouteDefinition.class);
            RouteDefinition definition = convertRouteDefinition(gatewayRouteDefinition);
            dynamicRouteService.add(definition);
        });
    }

    /**
     * 根据前端参数封装路由对象
     */
    private RouteDefinition convertRouteDefinition(GatewayRouteDefinition gwdefinition) {
        RouteDefinition definition = new RouteDefinition();
        definition.setId(gwdefinition.getId());
        definition.setOrder(gwdefinition.getOrder());
        //设置断言
        ArrayList<PredicateDefinition> pdList = new ArrayList<>();
        ArrayList<GatewayPredicateDefinition> gatewayPredicateDefinitionList = gwdefinition.getPredicates();
        gatewayPredicateDefinitionList.forEach(gpDefinition -> {
            PredicateDefinition predicate = new PredicateDefinition();
            predicate.setArgs(gpDefinition.getArgs());
            predicate.setName(gpDefinition.getName());
            pdList.add(predicate);
        });
        definition.setPredicates(pdList);
        //设置过滤器
        ArrayList<FilterDefinition> filters = new ArrayList();
        ArrayList<GatewayFilterDefinition> gatewayFilters = gwdefinition.getFilters();
        gatewayFilters.forEach(filterDefinition -> {
            FilterDefinition filter = new FilterDefinition();
            filter.setName(filterDefinition.getName());
            filter.setArgs(filterDefinition.getArgs());
            filters.add(filter);
        });
        definition.setFilters(filters);
        URI uri = null;
        if (gwdefinition.getUri().startsWith("http")) {
            uri = UriComponentsBuilder.fromHttpUrl(gwdefinition.getUri()).build().toUri();
        } else {
            uri = URI.create(gwdefinition.getUri());
        }
        definition.setUri(uri);
        return definition;
    }
}
